@@ -21,8 +21,8 @@ __intname__ = "command_runner"
 __author__ = "Orsiris de Jong"
 __copyright__ = "Copyright (C) 2015-2024 Orsiris de Jong for NetInvent SASU"
 __licence__ = "BSD 3 Clause"
-__version__ = "1.6.0"
-__build__ = "2024010401"
+__version__ = "1.7.0"
+__build__ = "2024091501"
 __compat__ = "python2.7+"
 
 import io
@@ -337,6 +337,9 @@ def to_encoding(
             except (ValueError, TypeError):
                 # What happens when str cannot be concatenated
                 logger.error("Output cannot be captured {}".format(process_output))
+    elif process_output is None:
+        # We deal with strings. Alter output string to avoid NoneType errors
+        process_output = ""
     return process_output
 
 
@@ -477,6 +480,7 @@ def command_runner(
     silent=False,  # type: bool
     priority=None,  # type: Union[int, str]
     io_priority=None,  # type: str
+    heartbeat=0,  # type: int
     **kwargs  # type: Any
 ):
     # type: (...) -> Union[Tuple[int, Optional[Union[bytes, str]]], Tuple[int, Optional[Union[bytes, str]], Optional[Union[bytes, str]]]]
@@ -508,6 +512,8 @@ def command_runner(
     priority and io_priority can be set to 'low', 'normal' or 'high'
     priority may also be an int from -20 to 20 on Unix
 
+    heartbeat will log a line every heartbeat seconds informing that we're still alive
+
     Returns a tuple (exit_code, output)
     """
 
@@ -643,6 +649,19 @@ def command_runner(
                 return output_stderr
             return None
 
+    def _heartbeat_thread(
+        process,  # type: Union[subprocess.Popen[str], subprocess.Popen]
+        heartbeat,  # type: int
+    ):
+        begin_time = datetime.now()
+        while True:
+            elapsed_time = int((datetime.now() - begin_time).total_seconds())
+            if elapsed_time > heartbeat and elapsed_time % heartbeat == 0:
+                logger.info("Still running command after %s seconds" % elapsed_time)
+            if process.poll() is not None:
+                break
+            sleep(1)
+
     def _poll_process(
         process,  # type: Union[subprocess.Popen[str], subprocess.Popen]
         timeout,  # type: int
@@ -670,7 +689,6 @@ def command_runner(
             Simple subfunction to check whether timeout is reached
             Since we check this alot, we put it into a function
             """
-
             if timeout and (datetime.now() - begin_time).total_seconds() > timeout:
                 kill_childs_mod(process.pid, itself=True, soft_kill=False)
                 raise TimeoutExpired(
@@ -681,6 +699,17 @@ def command_runner(
                 raise StopOnInterrupt(_get_error_output(output_stdout, output_stderr))
 
         begin_time = datetime.now()
+        if heartbeat:
+            heartbeat_thread = threading.Thread(
+                target=_heartbeat_thread,
+                args=(
+                    process,
+                    heartbeat,
+                ),
+            )
+            heartbeat_thread.daemon = True
+            heartbeat_thread.start()
+
         if encoding is False:
             output_stdout = output_stderr = b""
         else:
@@ -818,6 +847,16 @@ def command_runner(
         )
         thread.daemon = True  # was setDaemon(True) which has been deprecated
         thread.start()
+        if heartbeat:
+            heartbeat_thread = threading.Thread(
+                target=_heartbeat_thread,
+                args=(
+                    process,
+                    heartbeat,
+                ),
+            )
+            heartbeat_thread.daemon = True
+            heartbeat_thread.start()
 
         if encoding is False:
             output_stdout = output_stderr = b""
@@ -946,6 +985,7 @@ def command_runner(
                     logger.warning(
                         "Cannot set process priority {}. Access denied.".format(exc)
                     )
+                    logger.debug("Trace:", exc_info=True)
                 except Exception as exc:
                     logger.warning("Cannot set process priority: {}".format(exc))
                     logger.debug("Trace:", exc_info=True)
@@ -965,6 +1005,7 @@ def command_runner(
                             exc
                         )
                     )
+                    logger.debug("Trace:", exc_info=True)
                 except Exception as exc:
                     logger.warning("Cannot set io priority: {}".format(exc))
                     logger.debug("Trace:", exc_info=True)
@@ -1011,8 +1052,8 @@ def command_runner(
                 _stdout.write(output_stderr.encode(encoding, errors=errors))
 
         logger.debug(
-            'Command "{}" returned with exit code "{}". Command output was:'.format(
-                command, exit_code
+            'Command "{}" returned with exit code "{}". Command output was:\n{}'.format(
+                command, exit_code, to_encoding(output_stdout, error_encoding, errors)
             )
         )
     except subprocess.CalledProcessError as exc:
@@ -1060,7 +1101,7 @@ def command_runner(
         exit_code, output_stdout = (-253, message)
     except TimeoutExpired as exc:
         message = 'Timeout {} seconds expired for command "{}" execution. Original output was: {}'.format(
-            timeout, command, exc.output
+            timeout, command, to_encoding(exc.output, error_encoding, errors)[-1000:]
         )
         if not silent:
             logger.error(message)
@@ -1069,7 +1110,7 @@ def command_runner(
         exit_code, output_stdout = (-254, message)
     except StopOnInterrupt as exc:
         message = "Command {} was stopped because stop_on function returned True. Original output was: {}".format(
-            command, to_encoding(exc.output, error_encoding, errors)
+            command, to_encoding(exc.output, error_encoding, errors)[-1000:]
         )
         if not silent:
             logger.info(message)
